"""
config.py
Saves configuration in file as json file
"""
import json
import os

# path to save the configuration file
filename = os.path.join(os.environ["HOMEDRIVE"], os.environ["HOMEPATH"], "Documents", "fishy_config.json")


class Config:

    def __init__(self):
        """
        cache the configuration in a dict for faster access,
        if file is not found initialize the dict
        """
        self.config_dict = json.loads(open(filename).read()) if os.path.exists(filename) else dict()

    def get(self, key, default=None):
        """
        gets a value from  configuration,
        if it is not found, return the default configuration
        :param key: key of the config
        :param default: default value to return if key is not found
        :return: config value
        """
        return self.config_dict[key] if key in self.config_dict else default

    def set(self, key, value, save=True):
        """
        saves the configuration is cache (and saves it in file if needed)
        :param key: key to save
        :param value: value to save
        :param save: False if don't want to save right away
        """
        self.config_dict[key] = value
        if save:
            self.save_config()

    def delete(self, key):
        """
        deletes a key from config
        :param key: key to delete
        """
        del self.config_dict[key]
        self.save_config()

    def save_config(self):
        """
        save the cache to the file
        """
        with open(filename, 'w') as f:
            f.write(json.dumps(self.config_dict))




import requests
from whatsmyip.ip import get_ip
from whatsmyip.providers import GoogleDnsProvider

from fishy import helper
from . import urls
from .decorators import fallback, uses_session

_is_subbed = None
_session_id = None


@fallback(False)
def register_user(uid):
    ip = get_ip(GoogleDnsProvider)
    body = {"uid": uid, "ip": ip}
    response = requests.post(urls.user, json=body)
    return response.ok and response.json()["success"]


@fallback(None)
def send_notification(uid, message):
    if not is_subbed(uid):
        return False

    body = {"uid": uid, "message": message}
    requests.post(urls.notify, json=body)


@uses_session
@fallback(None)
def send_hole_deplete(uid, fish_caught, hole_time, fish_times):
    hole_data = {
        "fish_caught": fish_caught,
        "hole_time": hole_time,
        "fish_times": fish_times,
        "session": get_session(uid)
    }

    body = {"uid": uid, "hole_data": hole_data}
    requests.post(urls.hole_depleted, json=body)


@fallback(False)
def sub(uid, name):
    body = {"uid": uid, "discord_name": name}
    response = requests.post(urls.subscription, json=body)
    return response.json()["success"]


@fallback((False, False))
def is_subbed(uid, lazy=True):
    """
    :param uid:
    :param lazy:
    :return: Tuple[is_subbed, success]
    """
    global _is_subbed

    if lazy and _is_subbed is not None:
        return _is_subbed, True

    if uid is None:
        return False, False

    body = {"uid": uid}
    response = requests.get(urls.subscription, params=body)
    _is_subbed = response.json()["subbed"]
    return _is_subbed, True


@fallback(None)
def unsub(uid):
    global _is_subbed

    _is_subbed = False
    body = {"uid": uid}
    requests.delete(urls.subscription, json=body)


@fallback(None)
def get_session(config, lazy=True):
    global _session_id

    if lazy and _session_id is not None:
        return _session_id

    body = {"uid": config.get("uid")}
    response = requests.post(urls.session, params=body)

    if response.status_code == 405:
        config.delete("uid")
        helper.restart()
        return None

    _session_id = response.json()["session_id"]
    return _session_id



"""
auto_update.py
checks version and auto updates
"""
import logging
import re
import subprocess
import sys
import urllib.request
from os import execl

import pkg_resources
from bs4 import BeautifulSoup


def _normalize_version(v):
    """
    converts version string into an "normalized" of versions which is a list of version codes,
    eg, input: '0.3.0', output: [0,3,0]
    this is done so that, versions can be compared easily
    :param v: string
    :return: list
    """
    rv = []
    for x in v.split("."):
        try:
            rv.append(int(x))
        except ValueError:
            for y in re.split("([0-9]+)", x):
                try:
                    if y != '':
                        rv.append(int(y))
                except ValueError:
                    rv.append(y)
    return rv


def _get_highest_version(index, pkg):
    """
    Crawls web for latest version name then returns latest version
    :param index: website to check
    :param pkg: package name
    :return: latest version normalized
    """
    url = "{}/{}/".format(index, pkg)
    html = urllib.request.urlopen(url)
    if html.getcode() != 200:
        raise Exception  # not found
    soup = BeautifulSoup(html.read(), "html5lib")
    versions = []
    for link in soup.find_all('a'):
        text = link.get_text()
        try:
            version = re.search(pkg + '-(.*)\.tar\.gz', text).group(1)
            versions.append(_normalize_version(version))
        except AttributeError:
            pass
    if len(versions) == 0:
        raise Exception  # no version
    return max(versions)


def _get_current_version(pkg):
    """
    Gets the current version of the package installed
    :param pkg: name of the installed package
    :return: version normalized
    """
    return _normalize_version(pkg_resources.get_distribution(pkg).version)


def auto_upgrade():
    """
    public function,
    compares current version with the latest version (from web),
    if current version is older, then it updates and restarts the script
    """
    index = "https://pypi.python.org/simple"
    pkg = "fishy"
    hightest_version = _get_highest_version(index, pkg)
    if hightest_version > _get_current_version(pkg):
        logging.info(f"Updating to v{'.'.join(hightest_version)}, Please Wait...")
        subprocess.call(["python", '-m', 'pip', 'install', '--upgrade', 'fishy', '--user'])
        execl(sys.executable, *([sys.executable] + sys.argv))



class FishingMode:
    """
    State machine for fishing modes

    HValues         hue values for each fishing mode
    CurrentCount    number of times same hue color is read before it changes state
    CurrentMode     current mode of the state machine
    PrevMode        previous mode of the state machine
    FishingStarted  probably does nothing (not sure though)
    Modes           list of states
    """
    HValues = [60, 18, 100]

    CurrentMode = None
    PrevMode = None

    Modes = []

    def __init__(self, name, label, event):
        """
        create a new state
        :param name: name of the state
        :param label: integer, label of the state (int)
        :param event: object of class containing onEnterCallback & onExitCallback functions
                        which are called when state is changed
        """
        self.name = name
        self.label = label
        self.event = event

        FishingMode.Modes.append(self)

    @staticmethod
    def get_by_label(label):
        """
        find a state using label
        :param label: label integer
        :return: state
        """
        for m in FishingMode.Modes:
            if m.label == label:
                return m

    @staticmethod
    def loop(hue_values):
        """
        Executed in the start of the main loop in fishy.py
        Changes modes, calls mode events (callbacks) when mode is changed

        :param hue_values: hue_values read by the bot
        """
        if FishingMode.PrevMode is None:
            FishingMode.PrevMode = FishingMode.get_by_label(3)

        current_label = 3
        for i, val in enumerate(FishingMode.HValues):
            if hue_values == val:
                current_label = i

        FishingMode.CurrentMode = FishingMode.get_by_label(current_label)

        if FishingMode.CurrentMode != FishingMode.PrevMode:

            if FishingMode.PrevMode.event is not None:
                FishingMode.PrevMode.event.on_exit_callback(FishingMode.CurrentMode)

            if FishingMode.CurrentMode.event is not None:
                FishingMode.CurrentMode.event.on_enter_callback(FishingMode.PrevMode)

        FishingMode.PrevMode = FishingMode.CurrentMode



import typing
from abc import ABC, abstractmethod
from threading import Thread
from typing import Callable

if typing.TYPE_CHECKING:
    from fishy.gui import GUI


class IEngine(ABC):

    def __init__(self, config, gui_ref: 'Callable[[], GUI]'):
        self.get_gui = gui_ref
        self.start = False
        self.window = None
        self.thread = None
        self.config = config

    @property
    def gui(self):
        return self.get_gui().funcs

    def toggle_start(self):
        self.start = not self.start
        if self.start:
            self.thread = Thread(target=self.run)
            self.thread.start()

    @abstractmethod
    def run(self):
        ...



"""
tkinter HTML text widgets
"""
import sys
import tkinter as tk
from . import html_parser

VERSION = "0.1.0.post1"


class _ScrolledText(tk.Text):
    # ----------------------------------------------------------------------------------------------
    def __init__(self, master=None, **kw):
        self.frame = tk.Frame(master)

        self.vbar = tk.Scrollbar(self.frame)
        kw.update({'yscrollcommand': self.vbar.set})
        self.vbar.pack(side=tk.RIGHT, fill=tk.Y)
        self.vbar['command'] = self.yview

        tk.Text.__init__(self, self.frame, **kw)
        self.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        text_meths = vars(tk.Text).keys()
        methods = vars(tk.Pack).keys() | vars(tk.Grid).keys() | vars(tk.Place).keys()
        methods = methods.difference(text_meths)

        for m in methods:
            if m[0] != '_' and m != 'config' and m != 'configure':
                setattr(self, m, getattr(self.frame, m))

    def __str__(self):
        return str(self.frame)


class HTMLScrolledText(_ScrolledText):
    # ----------------------------------------------------------------------------------------------
    """
    HTML scrolled text widget
    """

    def __init__(self, *args, html=None, **kwargs):
        # ------------------------------------------------------------------------------------------
        super().__init__(*args, **kwargs)
        self._w_init(kwargs)
        self.html_parser = html_parser.HTMLTextParser()
        if isinstance(html, str):
            self.set_html(html)

    def _w_init(self, kwargs):
        # ------------------------------------------------------------------------------------------
        if not 'wrap' in kwargs.keys():
            self.config(wrap='word')
        if not 'background' in kwargs.keys():
            if sys.platform.startswith('win'):
                self.config(background='SystemWindow')
            else:
                self.config(background='white')

    def fit_height(self):
        # ------------------------------------------------------------------------------------------
        """
        Fit widget height to wrapped lines
        """
        for h in range(1, 4):
            self.config(height=h)
            self.master.update()
            if self.yview()[1] >= 1:
                break
        else:
            self.config(height=0.5 + 3 / self.yview()[1])

    def set_html(self, html, strip=True):
        # ------------------------------------------------------------------------------------------
        """
        Set HTML widget text. If strip is enabled (default) it ignores spaces and new lines.

        """
        prev_state = self.cget('state')
        self.config(state=tk.NORMAL)
        self.delete('1.0', tk.END)
        self.tag_delete(self.tag_names)
        self.html_parser.w_set_html(self, html, strip=strip)
        self.config(state=prev_state)


class HTMLText(HTMLScrolledText):
    # ----------------------------------------------------------------------------------------------
    """
    HTML text widget
    """

    def _w_init(self, kwargs):
        # ------------------------------------------------------------------------------------------
        super()._w_init(kwargs)
        self.vbar.pack_forget()

    def fit_height(self):
        # ------------------------------------------------------------------------------------------
        super().fit_height()
        # self.master.update()
        self.vbar.pack_forget()


class HTMLLabel(HTMLText):
    # ----------------------------------------------------------------------------------------------
    """
    HTML label widget
    """

    def _w_init(self, kwargs):
        # ------------------------------------------------------------------------------------------
        super()._w_init(kwargs)
        if not 'background' in kwargs.keys():
            if sys.platform.startswith('win'):
                self.config(background='SystemButtonFace')
            else:
                self.config(background='#d9d9d9')

        if not 'borderwidth' in kwargs.keys():
            self.config(borderwidth=0)

        if not 'padx' in kwargs.keys():
            self.config(padx=3)

    def set_html(self, *args, **kwargs):
        # ------------------------------------------------------------------------------------------
        super().set_html(*args, **kwargs)
        self.config(state=tk.DISABLED)



import logging

import cv2
import math
import win32gui
from win32api import GetSystemMetrics

import imutils
import numpy as np
from PIL import ImageGrab


class Window:
    """
    Records the game window, and allows to create instance to process it
    """
    Screen = None
    windowOffset = None
    titleOffset = None
    hwnd = None
    showing = False

    def __init__(self, crop=None, color=None, scale=None):
        """
        create a window instance with these pre process
        :param crop: [x1,y1,x2,y2] array defining the boundaries to crop
        :param color: color to use example cv2.COLOR_RGB2HSV
        :param scale: scaling the window
        """
        self.color = color
        self.crop = crop
        self.scale = scale

    @staticmethod
    def init(borderless: bool):
        """
        Executed once before the main loop,
        Finds the game window, and calculates the offset to remove the title bar
        """
        Window.hwnd = win32gui.FindWindow(None, "Elder Scrolls Online")
        rect = win32gui.GetWindowRect(Window.hwnd)
        client_rect = win32gui.GetClientRect(Window.hwnd)
        Window.windowOffset = math.floor(((rect[2] - rect[0]) - client_rect[2]) / 2)
        Window.titleOffset = ((rect[3] - rect[1]) - client_rect[3]) - Window.windowOffset
        if borderless:
            Window.titleOffset = 0

    @staticmethod
    def loop():
        """
        Executed in the start of the main loop
        finds the game window location and captures it
        """
        Window.showing = False

        bbox = (0, 0, GetSystemMetrics(0), GetSystemMetrics(1))

        temp_screen = np.array(ImageGrab.grab(bbox=bbox))

        temp_screen = cv2.cvtColor(temp_screen, cv2.COLOR_BGR2RGB)

        rect = win32gui.GetWindowRect(Window.hwnd)
        crop = (rect[0] + Window.windowOffset, rect[1] + Window.titleOffset, rect[2] - Window.windowOffset,
                rect[3] - Window.windowOffset)

        Window.Screen = temp_screen[crop[1]:crop[3], crop[0]:crop[2]]

        if Window.Screen.size == 0:
            logging.info("Don't minimize or drag game window outside the screen")
            return False

        return True

    @staticmethod
    def loop_end():
        """
        Executed in the end of the game loop
        """
        cv2.waitKey(25)

        if not Window.showing:
            cv2.destroyAllWindows()

    def get_capture(self):
        """
        copies the recorded screen and then pre processes its
        :return: game window image
        """
        temp_img = Window.Screen

        if self.color is not None:
            temp_img = cv2.cvtColor(temp_img, self.color)

        if self.crop is not None:
            temp_img = temp_img[self.crop[1]:self.crop[3], self.crop[0]:self.crop[2]]

        if self.scale is not None:
            temp_img = cv2.resize(temp_img, (self.scale[0], self.scale[1]), interpolation=cv2.INTER_AREA)

        return temp_img

    def processed_image(self, func=None):
        """
        processes the image using the function provided
        :param func: function to process image
        :return: processed image
        """
        if func is None:
            return self.get_capture()
        else:
            return func(self.get_capture())

    def show(self, name, resize=None, func=None, ready_img=None):
        """
        Displays the processed image for debugging purposes
        :param ready_img: send ready image, just show the `ready_img` directly
        :param name: unique name for the image, used to create a new window
        :param resize: scale the image to make small images more visible
        :param func: function to process the image
        """
        if ready_img is None:
            img = self.processed_image(func)

            if resize is not None:
                img = imutils.resize(img, width=resize)
        else:
            img = ready_img
        cv2.imshow(name, img)

        Window.showing = True



"""
HTML parser
"""
import os
import webbrowser
import tkinter as tk
from tkinter import font
from copy import deepcopy
from PIL import Image, ImageTk
from html.parser import HTMLParser
from collections import OrderedDict
import requests
from io import BytesIO


# __________________________________________________________________________________________________
class Defs:
    DEFAULT_TEXT_FONT_FAMILY = ("Segoe ui", "Calibri", "Helvetica", "TkTextFont")
    FONT_SIZE = 14
    PREFORMATTED_FONT_FAMILY = ("Courier", "DejaVu Sans Mono", "TkFixedFont")
    HEADINGS_FONT_SIZE = {
        'h1': 32,
        'h2': 24,
        'h3': 18,
        'h4': 16,
        'h5': 13,
        'h6': 10,
    }


class HTML:
    # ----------------------------------------------------------------------------------------------
    """
    List of supported HTML tags and attrs 
    """

    class Tag():
        BR = 'br'
        UL = 'ul'
        OL = 'ol'
        LI = 'li'
        IMG = 'img'
        A = 'a'
        B = 'b'
        STRONG = 'strong'
        I = 'i'
        EM = 'em'
        U = 'u'
        MARK = 'mark'
        SPAN = 'span'
        DIV = 'div'
        P = 'p'
        PRE = 'pre'
        CODE = 'code'
        H1 = 'h1'
        H2 = 'h2'
        H3 = 'h3'
        H4 = 'h4'
        H5 = 'h5'
        H6 = 'h6'

    class Attrs():
        STYLE = 'style'
        HREF = 'href'
        SRC = 'src'
        WIDTH = 'width'
        HEIGHT = 'height'
        TYPE = 'type'

    class TypeOrderedList():
        _1 = '1'
        a = 'a'
        A = 'A'

    class Style():
        COLOR = 'color'
        BACKGROUD_COLOR = 'background-color'
        FONT_FAMILY = 'font-family'
        FONT_SIZE = 'font-size'
        TEXT_ALIGN = 'text-align'
        TEXT_DECORATION = 'text-decoration'

    class StyleTextDecoration():
        UNDERLINE = 'underline'
        LINE_THROUGH = 'line-through'

    HEADING_TAGS = (
        Tag.H1,
        Tag.H2,
        Tag.H3,
        Tag.H4,
        Tag.H5,
        Tag.H6,
    )

    TEXT_ALIGN_TAGS = HEADING_TAGS + (
        Tag.UL,
        Tag.OL,
        Tag.LI,
        Tag.DIV,
        Tag.P,
        Tag.PRE,
        Tag.CODE,
    )

    NEW_LINE_TAGS = HEADING_TAGS + (
        Tag.UL,
        Tag.OL,
        Tag.DIV,
        Tag.P,
        Tag.PRE,
        Tag.CODE,
    )

    STYLE_TAGS = TEXT_ALIGN_TAGS + (
        Tag.A,
        Tag.B,
        Tag.STRONG,
        Tag.I,
        Tag.EM,
        Tag.U,
        Tag.MARK,
        Tag.SPAN,
    )


# --------------------------------------------------------------------------------------------------
# Text widget defs

class WCfg():
    KEY = "config"
    BACKGROUND = "background"
    FOREGROUND = "foreground"
    JUSTIFY = "justify"
    TABS = "tabs"


class Fnt():
    KEY = "font"
    FAMILY = "family"
    SIZE = "size"
    WEIGHT = "weight"
    SLANT = "slant"
    UNDERLINE = "underline"
    OVERSTRIKE = "overstrike"


class Bind():
    KEY = "bind"
    LINK = "link"
    IMAGE = "image"


class WTag():
    START_INDEX = "start_index"
    END_INDEX = "end_index"


DEFAULT_STACK = {
    WCfg.KEY: {
        WCfg.BACKGROUND: [],
        WCfg.FOREGROUND: [("__DEFAULT__", "black")],
        WCfg.JUSTIFY: [("__DEFAULT__", 'left')],
        WCfg.TABS: [("__DEFAULT__", ())],
    },
    Fnt.KEY: {
        Fnt.FAMILY: [],
        Fnt.SIZE: [("__DEFAULT__", Defs.FONT_SIZE)],
        Fnt.WEIGHT: [("__DEFAULT__", 'normal')],
        Fnt.SLANT: [("__DEFAULT__", 'roman')],
        Fnt.UNDERLINE: [("__DEFAULT__", False)],
        Fnt.OVERSTRIKE: [("__DEFAULT__", False)],
    },
    Bind.KEY: {
        Bind.LINK: [("__DEFAULT__", None)],
    },
}


# __________________________________________________________________________________________________
# functions
def get_existing_font(font_families):
    # ------------------------------------------------------------------------------------------
    try:
        return next(filter(lambda f: f.lower() in (f.lower() for f in font.families()), font_families))
    except:
        return "TkTextFont"


# __________________________________________________________________________________________________
# classes
class HLinkSlot():
    # ----------------------------------------------------------------------------------------------

    def __init__(self, w, tag_name, url):
        # ------------------------------------------------------------------------------------------
        self._w = w
        self.tag_name = tag_name
        self.URL = url

    def call(self, event):
        # ------------------------------------------------------------------------------------------
        webbrowser.open(self.URL)
        self._w.tag_config(self.tag_name, foreground="purple")

    def enter(self, event):
        # ------------------------------------------------------------------------------------------
        self._w.config(cursor="hand2")

    def leave(self, event):
        # ------------------------------------------------------------------------------------------
        self._w.config(cursor="")


class ListTag():
    # ----------------------------------------------------------------------------------------------
    def __init__(self, ordered: bool, list_type=None):
        # ------------------------------------------------------------------------------------------
        self.ordered = ordered
        self.type = list_type
        self.index = 0

    def add(self):
        # ------------------------------------------------------------------------------------------
        if self.ordered:
            self.index += 1

    def line_index(self):
        # ------------------------------------------------------------------------------------------
        if self.ordered:
            if self.type == HTML.TypeOrderedList._1:
                return str(self.index)
            elif self.type == HTML.TypeOrderedList.a:
                return self._index_to_str(self.index).lower()
            elif self.type == HTML.TypeOrderedList.A:
                return self._index_to_str(self.index).upper()
        else:
            return chr(8226)

    def _index_to_str(self, index):
        # ------------------------------------------------------------------------------------------
        prefix = ""
        if index > 26:
            prefix = self._index_to_str(index // 26)
            index = index % 26

        return prefix + chr(0x60 + index)


class HTMLTextParser(HTMLParser):
    # ----------------------------------------------------------------------------------------------

    def __init__(self):
        # ------------------------------------------------------------------------------------------
        super().__init__()
        # set list tabs
        self.cached_images = {}

        self.DEFAULT_TEXT_FONT_FAMILY = get_existing_font(Defs.DEFAULT_TEXT_FONT_FAMILY)
        self.PREFORMATTED_FONT_FAMILY = get_existing_font(Defs.PREFORMATTED_FONT_FAMILY)

    def _parse_attrs(self, attrs):
        # ------------------------------------------------------------------------------------------
        attrs_dict = {
            HTML.Attrs.STYLE: {},
            HTML.Attrs.HREF: None,
            HTML.Attrs.SRC: None,
            HTML.Attrs.WIDTH: None,
            HTML.Attrs.HEIGHT: None,
            HTML.Attrs.TYPE: None,
        }
        for k, v in attrs:
            k = k.lower()
            if k == HTML.Attrs.STYLE:
                for p in v.split(";"):
                    try:
                        p_key = p.split(":")[0].strip().lower()
                        p_value = p.split(":")[1].strip().lower()
                        attrs_dict[HTML.Attrs.STYLE][p_key] = p_value
                    except:
                        pass
            elif k in (HTML.Attrs.HREF, HTML.Attrs.SRC, HTML.Attrs.WIDTH, HTML.Attrs.HEIGHT, HTML.Attrs.TYPE):
                attrs_dict[k] = v
        return attrs_dict

    def _w_tags_add(self):
        # ------------------------------------------------------------------------------------------
        tag = {
            WCfg.KEY: {},
            Fnt.KEY: {},
            Bind.KEY: {}
        }

        for k1 in (WCfg.KEY, Fnt.KEY, Bind.KEY):
            for k2 in DEFAULT_STACK[k1]:
                tag[k1][k2] = self.stack[k1][k2][-1][1]

        self._w_tags[self._w.index("end-1c")] = tag

    def _stack_get_main_key(self, key):
        # ------------------------------------------------------------------------------------------
        if key in WCfg.__dict__.values():
            main_key = WCfg.KEY
        elif key in Fnt.__dict__.values():
            main_key = Fnt.KEY
        elif key in Bind.__dict__.values():
            main_key = Bind.KEY
        else:
            raise ValueError("key %s doesn't exists" % key)

        return main_key

    def _stack_add(self, tag, key, value=None):
        # ------------------------------------------------------------------------------------------
        main_key = self._stack_get_main_key(key)

        if value is None:
            # if value is none, add the previous value
            value = self.stack[main_key][key][-1][1]

        self.stack[main_key][key].append((tag, value))

    def _stack_index(self, tag, key):
        # ------------------------------------------------------------------------------------------
        main_key = self._stack_get_main_key(key)
        index = None
        for i, v in enumerate(self.stack[main_key][key]):
            if v[0] == tag:
                index = i

        return index

    def _stack_pop(self, tag, key):
        # ------------------------------------------------------------------------------------------
        main_key = self._stack_get_main_key(key)

        index = None
        if len(self.stack[main_key][key]) > 1:
            index = self._stack_index(tag, key)

        if index is not None:
            return self.stack[main_key][key].pop(index)[1]

    def _parse_styles(self, tag, attrs):
        # ------------------------------------------------------------------------------------------
        # -------------------------------------------------------------------------------- [ COLOR ]
        if HTML.Style.COLOR in attrs[HTML.Attrs.STYLE].keys():
            self._stack_add(tag, WCfg.FOREGROUND, attrs[HTML.Attrs.STYLE][HTML.Style.COLOR])
        elif tag == HTML.Tag.A and attrs[HTML.Attrs.HREF]:
            self._stack_add(tag, WCfg.FOREGROUND, "blue")
        else:
            self._stack_add(tag, WCfg.FOREGROUND)

        # ---------------------------------------------------------------------- [ BACKGROUD_COLOR ]
        if HTML.Style.BACKGROUD_COLOR in attrs[HTML.Attrs.STYLE].keys():
            self._stack_add(tag, WCfg.BACKGROUND, attrs[HTML.Attrs.STYLE][HTML.Style.BACKGROUD_COLOR])
        elif tag == HTML.Tag.MARK:
            self._stack_add(tag, WCfg.BACKGROUND, "yellow")
        else:
            self._stack_add(tag, WCfg.BACKGROUND)

        # -------------------------------------------------------------------------- [ FONT_FAMILY ]
        # font family
        if HTML.Style.FONT_FAMILY in attrs[HTML.Attrs.STYLE].keys():
            font_family = Defs.DEFAULT_TEXT_FONT_FAMILY
            for f in attrs[HTML.Attrs.STYLE][HTML.Style.FONT_FAMILY].split(","):
                f = f.strip()
                if f in map(lambda f: f.lower(), font.families()):
                    font_family = f
                    break
            self._stack_add(tag, Fnt.FAMILY, font_family)
        elif tag in (HTML.Tag.PRE, HTML.Tag.CODE):
            self._stack_add(tag, Fnt.FAMILY, self.PREFORMATTED_FONT_FAMILY)
        else:
            self._stack_add(tag, Fnt.FAMILY)

        # ---------------------------------------------------------------------------- [ FONT_SIZE ]
        if HTML.Style.FONT_SIZE in attrs[HTML.Attrs.STYLE].keys():
            size = Defs.FONT_SIZE
            if attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE].endswith("px"):
                if attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE][:-2].isdigit():
                    size = int(attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE][:-2])
            elif attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE].endswith(r"%"):
                if attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE][:-1].isdigit():
                    size = int((int(attrs[HTML.Attrs.STYLE][HTML.Style.FONT_SIZE][:-1]) * Defs.FONT_SIZE) / 100)
            self._stack_add(tag, Fnt.SIZE, size)
        elif tag.startswith('h') and len(tag) == 2:
            self._stack_add(tag, Fnt.SIZE, Defs.HEADINGS_FONT_SIZE[tag])
        else:
            self._stack_add(tag, Fnt.SIZE)

        # --------------------------------------------------------------------------- [ TEXT_ALIGN ]
        if HTML.Style.TEXT_ALIGN in attrs[HTML.Attrs.STYLE].keys() and tag in HTML.TEXT_ALIGN_TAGS:
            self._stack_add(tag, WCfg.JUSTIFY, attrs[HTML.Attrs.STYLE][HTML.Style.TEXT_ALIGN])
        else:
            self._stack_add(tag, WCfg.JUSTIFY)

        # ---------------------------------------------------------------------- [ TEXT_DECORATION ]
        if HTML.Style.TEXT_DECORATION in attrs[HTML.Attrs.STYLE].keys():
            if tag == HTML.Tag.STRONG:
                self._stack_add(tag, Fnt.UNDERLINE, False)
                self._stack_add(tag, Fnt.OVERSTRIKE, False)
            elif HTML.StyleTextDecoration.UNDERLINE in attrs[HTML.Attrs.STYLE][HTML.Style.TEXT_DECORATION]:
                self._stack_add(tag, Fnt.UNDERLINE, True)
                self._stack_add(tag, Fnt.OVERSTRIKE, False)
            elif HTML.StyleTextDecoration.LINE_THROUGH in attrs[HTML.Attrs.STYLE][HTML.Style.TEXT_DECORATION]:
                self._stack_add(tag, Fnt.UNDERLINE, False)
                self._stack_add(tag, Fnt.OVERSTRIKE, True)
            else:
                self._stack_add(tag, Fnt.UNDERLINE)
                self._stack_add(tag, Fnt.OVERSTRIKE)
        else:
            if tag == HTML.Tag.A and attrs[HTML.Attrs.HREF]:
                self._stack_add(tag, Fnt.UNDERLINE, True)
                self._stack_add(tag, Fnt.OVERSTRIKE, False)
            elif tag == HTML.Tag.U:
                self._stack_add(tag, Fnt.UNDERLINE, True)
                self._stack_add(tag, Fnt.OVERSTRIKE, False)
            else:
                self._stack_add(tag, Fnt.UNDERLINE)
                self._stack_add(tag, Fnt.OVERSTRIKE)

    def handle_starttag(self, tag, attrs):
        # ------------------------------------------------------------------------------------------
        tag = tag.lower()
        attrs = self._parse_attrs(attrs)

        if tag in HTML.STYLE_TAGS:
            # ---------------------------------------------------------------------- [ STYLED_TAGS ]
            self._parse_styles(tag, attrs)

            if tag == HTML.Tag.B or tag == HTML.Tag.STRONG or tag in HTML.HEADING_TAGS:
                self._stack_add(tag, Fnt.WEIGHT, "bold")

            elif tag == HTML.Tag.I or tag == HTML.Tag.EM:
                self._stack_add(tag, Fnt.SLANT, "italic")

            elif tag == HTML.Tag.A:
                self._stack_add(tag, Bind.LINK, attrs[HTML.Attrs.HREF])

            elif tag == HTML.Tag.OL:
                # ---------------------------------------------------------------- [ ORDERED_LISTS ]
                if attrs[HTML.Attrs.TYPE] and attrs[HTML.Attrs.TYPE] in HTML.TypeOrderedList.__dict__.values():
                    list_type = attrs[HTML.Attrs.TYPE]
                else:
                    list_type = HTML.TypeOrderedList._1
                self.list_tags.append(ListTag(ordered=True, list_type=list_type))

                tabs = []
                for i in range(len(self.list_tags)):
                    offset = 30 * (i + 1)
                    tabs += [offset, tk.RIGHT, offset + 5, tk.LEFT]
                self._stack_add(tag, WCfg.TABS, tabs)

            elif tag == HTML.Tag.UL:
                # -------------------------------------------------------------- [ UNORDERED_LISTS ]
                self.list_tags.append(ListTag(ordered=False))

                tabs = []
                for i in range(len(self.list_tags)):
                    offset = 30 * (i + 1)
                    tabs += [offset, tk.RIGHT, offset + 5, tk.LEFT]
                self._stack_add(tag, WCfg.TABS, tabs)

            elif tag == HTML.Tag.LI:
                # ------------------------------------------------------------------ [ LISTS_LINES ]
                level = len(self.list_tags)
                if level:
                    self.list_tags[-1].add()

                    if self.strip:
                        self._insert_new_line()

                    line_index = self.list_tags[-1].line_index()
                    if self.list_tags[-1].ordered:
                        line_index = "\t" + "\t\t" * (level - 1) + line_index + ".\t"
                    else:
                        line_index = "\t" + "\t\t" * (level - 1) + line_index + "\t"

                    self._stack_add(tag, Fnt.UNDERLINE, False)
                    self._stack_add(tag, Fnt.OVERSTRIKE, False)
                    self._w_tags_add()
                    self._w.insert(tk.INSERT, line_index)
                    self._stack_pop(tag, Fnt.UNDERLINE)
                    self._stack_pop(tag, Fnt.OVERSTRIKE)

        elif tag == HTML.Tag.IMG and attrs[HTML.Attrs.SRC]:
            # -------------------------------------------------------------------- [ UNSTYLED_TAGS ]
            image = None
            print(attrs[HTML.Attrs.SRC], self.cached_images)
            if attrs[HTML.Attrs.SRC].startswith(("https://", "ftp://", "http://")):
                if attrs[HTML.Attrs.SRC] in self.cached_images.keys():
                    image = deepcopy(self.cached_images[attrs[HTML.Attrs.SRC]])
                else:
                    try:
                        image = Image.open(BytesIO(requests.get(attrs[HTML.Attrs.SRC]).content))
                        self.cached_images[attrs[HTML.Attrs.SRC]] = deepcopy(image)
                    except:
                        pass

            if attrs[HTML.Attrs.SRC] in self.cached_images.keys():
                image = deepcopy(self.cached_images[attrs[HTML.Attrs.SRC]])
            elif os.path.exists(attrs[HTML.Attrs.SRC]):
                image = Image.open(attrs[HTML.Attrs.SRC])
                self.cached_images[attrs[HTML.Attrs.SRC]] = deepcopy(image)
            if image:
                width = image.size[0]
                height = image.size[1]
                resize = False
                if str(attrs[HTML.Attrs.WIDTH]).isdigit():
                    width = int(attrs[HTML.Attrs.WIDTH])
                    resize = True
                if str(attrs[HTML.Attrs.HEIGHT]).isdigit():
                    height = int(attrs[HTML.Attrs.HEIGHT])
                    resize = True
                if resize:
                    image = image.resize((width, height), Image.ANTIALIAS)
                self.images.append(ImageTk.PhotoImage(image))
                self._w.image_create(tk.INSERT, image=self.images[-1])

        if self.strip:
            # ------------------------------------------------------------------------ [ NEW_LINES ]
            if tag == HTML.Tag.BR:
                self._insert_new_line()
            else:
                self.html_tags.append(tag)

            if tag in HTML.NEW_LINE_TAGS and self.strip and self._w.index("end-1c") != "1.0":
                if tag in (HTML.Tag.DIV,):
                    self._insert_new_line()
                elif tag in (HTML.Tag.UL, HTML.Tag.OL):
                    if len(self.list_tags) == 1:
                        self._insert_new_line(double=True)
                    else:
                        self._insert_new_line(double=False)
                else:
                    self._insert_new_line(double=True)

        self._w_tags_add()

    def handle_charref(self, data):
        # ------------------------------------------------------------------------------------------
        try:
            char = chr(int(data))
            self._w.insert(tk.INSERT, char)
        except:
            pass

    def _insert_new_line(self, double=False):
        # ------------------------------------------------------------------------------------------
        self._remove_last_space()
        if self._w.get("end-3c", "end-1c") == "\n\n":
            pass
        elif self._w.get("end-2c", "end-1c") == "\n":
            if double:
                self._w.insert(tk.INSERT, "\n")
        else:
            if double:
                self._w.insert(tk.INSERT, "\n\n")
            else:
                self._w.insert(tk.INSERT, "\n")

    def _text_rstrip(self):
        # ------------------------------------------------------------------------------------------
        for _ in range(3):
            if self._w.get("end-2c", "end-1c") in (" ", "\n"):
                self._w.delete("end-2c", "end-1c")

    def _remove_last_space(self):
        # ------------------------------------------------------------------------------------------
        if self._w.get("end-2c", "end-1c") == " ":
            self._w.delete("end-2c", "end-1c")

    def _remove_multi_spaces(self, data):
        # ------------------------------------------------------------------------------------------
        data = data.replace("  ", " ")
        if "  " in data:
            data = self._remove_multi_spaces(data)
        return data

    def handle_data(self, data):
        # ------------------------------------------------------------------------------------------
        if self.strip:
            if len(self.html_tags) and self.html_tags[-1] in (HTML.Tag.PRE, HTML.Tag.CODE):
                pass
            elif not data.strip():
                data = ""
            else:
                # left strip
                if self._w.index("end-1c").endswith(".0"):
                    data = data.lstrip()
                elif self._w.get("end-2c", "end-1c") == " ":
                    data = data.lstrip()

                data = data.replace("\n", " ").replace("\t", " ")
                data = data + " "
                data = self._remove_multi_spaces(data)
                if len(self.html_tags):
                    level = len(self.list_tags)
                    if self.html_tags[-1] in (HTML.Tag.UL, HTML.Tag.OL):
                        self._w.insert(tk.INSERT, "\t" * 2 * level)

        self._w.insert(tk.INSERT, data)

    def handle_endtag(self, tag):
        # ------------------------------------------------------------------------------------------
        tag = tag.lower()

        try:
            index = len(self.html_tags) - self.html_tags[::-1].index(tag) - 1
            self.html_tags.pop(index)
        except:
            pass

        if tag in HTML.STYLE_TAGS:

            self._stack_pop(tag, WCfg.FOREGROUND)
            self._stack_pop(tag, WCfg.BACKGROUND)
            self._stack_pop(tag, WCfg.JUSTIFY)
            self._stack_pop(tag, Fnt.FAMILY)
            self._stack_pop(tag, Fnt.SIZE)
            self._stack_pop(tag, Fnt.UNDERLINE)
            self._stack_pop(tag, Fnt.OVERSTRIKE)

            if tag == HTML.Tag.B or tag == HTML.Tag.STRONG or tag in HTML.HEADING_TAGS:
                self._stack_pop(tag, Fnt.WEIGHT)

            elif tag == HTML.Tag.I or tag == HTML.Tag.EM:
                self._stack_pop(tag, Fnt.SLANT)

            elif tag == HTML.Tag.A:
                self._stack_pop(tag, Bind.LINK)

            elif tag == HTML.Tag.OL or tag == HTML.Tag.UL:
                if len(self.list_tags):
                    self.list_tags = self.list_tags[:-1]

                self._stack_pop(tag, WCfg.TABS)

        if tag in HTML.NEW_LINE_TAGS and self.strip:
            self._insert_new_line()

        self._w_tags_add()

        if tag in HTML.NEW_LINE_TAGS and self.strip:
            if tag in (HTML.Tag.DIV, HTML.Tag.UL, HTML.Tag.OL):
                if not len(self.list_tags):
                    self._insert_new_line(double=True)
            else:
                self._insert_new_line(double=True)

    def _w_tags_apply_all(self):
        # ------------------------------------------------------------------------------------------
        # update indexes
        if self.strip:
            self._text_rstrip()
        end_index = tk.END
        for key, tag in reversed(tuple(self._w_tags.items())):
            tag[WTag.START_INDEX] = key
            tag[WTag.END_INDEX] = end_index
            end_index = key

        # add tags
        self.hlink_slots = []
        for key, tag in self._w_tags.items():
            self._w.tag_add(key, tag[WTag.START_INDEX], tag[WTag.END_INDEX])
            self._w.tag_config(key, font=font.Font(**tag[Fnt.KEY]), **tag[WCfg.KEY])
            if tag[Bind.KEY][Bind.LINK]:
                self.hlink_slots.append(HLinkSlot(self._w, key, tag[Bind.KEY][Bind.LINK]))
                self._w.tag_bind(key, "<Button-1>", self.hlink_slots[-1].call)
                self._w.tag_bind(key, "<Leave>", self.hlink_slots[-1].leave)
                self._w.tag_bind(key, "<Enter>", self.hlink_slots[-1].enter)

    def w_set_html(self, w, html, strip):
        # ------------------------------------------------------------------------------------------
        self._w = w
        self.stack = deepcopy(DEFAULT_STACK)
        self.stack[WCfg.KEY][WCfg.BACKGROUND].append(("__DEFAULT__", self._w.cget("background")))
        self.stack[Fnt.KEY][Fnt.FAMILY].append(("__DEFAULT__", self.DEFAULT_TEXT_FONT_FAMILY))
        self._w_tags = OrderedDict()
        self.html_tags = []
        self.images = []
        self.list_tags = []
        self.strip = strip
        self._w_tags_add()
        self.feed(html)
        self._w_tags_apply_all()
        del self._w



from .auto_update import auto_upgrade
from .config import Config
from .helper import open_web, initialize_uid, install_thread_excepthook, unhandled_exception_logging, manifest_file, \
    create_shortcut_first, check_addon, restart, create_shortcut, not_implemented



import math
import cv2
import logging
import time

import numpy as np
import pywintypes
import pytesseract

from fishy.engine.IEngine import IEngine
from fishy.engine.window import Window
from pynput import keyboard, mouse
from pynput.keyboard import Key

mse = mouse.Controller()
kb = keyboard.Controller()


def get_crop_coods(window):
    Window.loop()
    img = window.get_capture()
    img = cv2.inRange(img, 0, 1)
    Window.loop_end()

    cnt, h = cv2.findContours(img, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    """
    code from https://stackoverflow.com/a/45770227/4512396
    """
    for i in range(len(cnt)):
        area = cv2.contourArea(cnt[i])
        if 5000 < area < 100000:
            mask = np.zeros_like(img)
            cv2.drawContours(mask, cnt, i, 255, -1)
            x, y, w, h = cv2.boundingRect(cnt[i])
            return x, y, x + w, y + h


def image_pre_process(img):
    scale_percent = 200  # percent of original size
    width = int(img.shape[1] * scale_percent / 100)
    height = int(img.shape[0] * scale_percent / 100)
    dim = (width, height)
    img = cv2.resize(img, dim, interpolation=cv2.INTER_AREA)
    img = cv2.bitwise_not(img)
    return img


# noinspection PyBroadException
def get_values_from_image(img, tesseract_dir):
    try:
        pytesseract.pytesseract.tesseract_cmd = tesseract_dir+'/tesseract.exe'
        tessdata_dir_config = f'--tessdata-dir "{tesseract_dir}" -c tessedit_char_whitelist=0123456789.'

        text = pytesseract.image_to_string(img, lang="eng", config=tessdata_dir_config)
        vals = text.split(":")
        return float(vals[0]), float(vals[1]), float(vals[2])
    except Exception:
        logging.error("Couldn't read coods")
        return None


class FullAuto(IEngine):
    def __init__(self, config, gui_ref):
        super().__init__(config, gui_ref)
        self.factors = None
        self.tesseract_dir = None

    def run(self):

        try:
            Window.init(False)
        except pywintypes.error:
            logging.info("Game window not found")
            self.toggle_start()
            return

        self.window = Window(color=cv2.COLOR_RGB2GRAY)
        self.window.crop = get_crop_coods(self.window)
        self.tesseract_dir = self.config.get("tesseract_dir", None)

        if self.tesseract_dir is None:
            logging.warning("Can't start without Tesseract Directory")
            self.gui.bot_started(False)
            self.toggle_start()
            return

        if self.get_gui is not None:
            self.gui.bot_started(True)

        logging.info("Controlls:\nUP: Callibrate\nLEFT: Pring Coordinates\nDOWN: Quit")
        with keyboard.Listener(
                on_press=self.on_press,
        ):
            while self.start:
                Window.loop()

                self.window.show("test", func=image_pre_process)
                Window.loop_end()
        self.gui.bot_started(False)

    def get_coods(self):
        return get_values_from_image(self.window.processed_image(func=image_pre_process), self.tesseract_dir)

    def callibrate(self):
        logging.debug("Callibrating...")
        walking_time = 3
        rotate_by = 100

        coods = self.get_coods()
        if coods is None:
            return

        x1, y1, rot1 = coods

        kb.press('w')
        time.sleep(walking_time)
        kb.release('w')
        time.sleep(0.5)

        coods = self.get_coods()
        if coods is None:
            return
        x2, y2, rot2 = coods

        mse.move(rotate_by, 0)
        time.sleep(0.5)

        coods = self.get_coods()
        if coods is None:
            return
        x3, y3, rot3 = coods

        if rot3 > rot2:
            rot3 -= 360

        rot_factor = (rot3 - rot2) / rotate_by
        move_factor = math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2) / walking_time
        self.factors = move_factor, rot_factor
        logging.info(self.factors)

    def on_press(self, key):
        if key == Key.down:
            bot.start = False
            self.toggle_start()
            if self.get_gui is not None:
                quit()
        elif key == Key.left:
            logging.info(self.get_coods())
        elif key == Key.up:
            self.callibrate()


if __name__ == '__main__':
    logging.getLogger("").setLevel(logging.DEBUG)
    # noinspection PyTypeChecker
    bot = FullAuto(None, None)
    bot.toggle_start()
    with keyboard.Listener(
            on_press=bot.on_press,
    ) as listener:
        listener.join()



import time
from tkinter import *
from tkinter import messagebox
from tkinter.ttk import *

from fishy import web
import typing

from fishy.libs.tkhtmlview import HTMLLabel

if typing.TYPE_CHECKING:
    from . import GUI


# noinspection PyProtectedMember
def _give_notification_link(gui: 'GUI'):
    if web.is_subbed(gui._config.get("uid"))[0]:
        web.unsub(gui._config.get("uid"))
        return

    # set notification checkbutton
    gui._notify.set(0)

    def quit_top():
        top.destroy()
        top_running[0] = False

    def check():
        if web.sub(gui._config.get("uid"), discord_name.get()):
            if web.is_subbed(gui._config.get("uid"), False)[0]:
                gui._notify.set(1)
                messagebox.showinfo("Note!", "Notification configured successfully!")
                quit_top()
        else:
            messagebox.showerror("Error", "Subscription wasn't successful")

    top_running = [True]

    top = Toplevel(background=gui._root["background"])
    top.minsize(width=300, height=300)
    top.title("Notification Setup")

    html_label = HTMLLabel(top,
                           html=f'<div style="color: {gui._console["fg"]}; text-align: center">'
                                f'<p><span style="font-size:20px">Step 1.</span><br/>'
                                f'Join <a href="https://discord.definex.in/">Discord server</a></p>'
                                f'<p><span style="font-size:20px">Step 2.</span><br/>'
                                f'Enter username (ex. Fishy#1234)'
                                f'</div>', background=gui._root["background"])

    html_label.pack(pady=(20, 5))
    html_label.fit_height()

    discord_name = Entry(top, justify=CENTER, font="Calibri 15")
    discord_name.pack(padx=(15, 15), expand=True, fill=BOTH)

    html_label = HTMLLabel(top,
                           html=f'<div style="color: {gui._console["fg"]}; text-align: center">'
                                f'<p><span style="font-size:20px">Step 3.</span><br/>'
                                f'Install Discord App on your phone</p>'
                                f'<p><span style="font-size:20px">Step 4.</span><br/></p>'
                                f'</div>', background=gui._root["background"])

    html_label.pack(pady=(5, 5))
    html_label.fit_height()

    Button(top, text="REGISTER", command=check).pack(pady=(5, 20))

    top.protocol("WM_DELETE_WINDOW", quit_top)
    top.grab_set()
    while top_running[0]:
        top.update()
        time.sleep(0.01)
    top.grab_release()



import time
import typing
from collections import Callable
from threading import Thread

import cv2
import logging

import pywintypes

from fishy.engine.IEngine import IEngine
from fishy.engine.semifisher import fishing_event
from .fishing_event import HookEvent, StickEvent, LookEvent, IdleEvent
from .fishing_mode import FishingMode
from .pixel_loc import PixelLoc
from fishy.engine.window import Window

if typing.TYPE_CHECKING:
    from fishy.gui import GUI


class SemiFisherEngine(IEngine):
    def __init__(self, config, gui_ref: 'Callable[[], GUI]'):
        super().__init__(config, gui_ref)
        self.fishPixWindow = None

    def run(self):
        """
        Starts the fishing
        code explained in comments in detail
        """

        action_key = self.config.get("action_key", "e")
        borderless = self.config.get("borderless", False)

        # initialize widow
        # noinspection PyUnresolvedReferences
        try:
            Window.init(borderless)
        except pywintypes.error:
            logging.info("Game window not found")
            self.start = False
            return

        # initializes fishing modes and their callbacks
        FishingMode("hook", 0, HookEvent(action_key, False))
        FishingMode("stick", 1, StickEvent())
        FishingMode("look", 2, LookEvent(action_key))
        FishingMode("idle", 3, IdleEvent(self.config.get("uid")))

        self.fishPixWindow = Window(color=cv2.COLOR_RGB2HSV)

        # check for game window and stuff
        self.gui.bot_started(True)
        logging.info("Starting the bot engine, look at the fishing hole to start fishing")
        Thread(target=self._wait_and_check).start()
        while self.start:
            # Services to be ran in the start of the main loop
            success = Window.loop()

            if not success:
                self.gui.bot_started(False)
                self.toggle_start()
                continue

            # get the PixelLoc and find the color values, to give it to `FishingMode.Loop`
            self.fishPixWindow.crop = PixelLoc.val
            hue_value = self.fishPixWindow.get_capture()[0][0][0]
            FishingMode.loop(hue_value)
            # Services to be ran in the end of the main loop
            Window.loop_end()
        logging.info("Fishing engine stopped")
        self.gui.bot_started(False)

    def _wait_and_check(self):
        time.sleep(10)
        if not fishing_event._FishingStarted and self.start:
            self.gui.show_error("Doesn't look like fishing has started\n\n"
                                "Check out #read-me-first on our discord channel to troubleshoot the issue")

    def show_pixel_vals(self):
        def show():
            freq = 0.5
            t = 0
            while t < 10.0:
                t += freq
                logging.debug(str(FishingMode.CurrentMode.label) + ":" + str(self.fishPixWindow.get_capture()[0][0]))
                time.sleep(freq)

        logging.debug("Will display pixel values for 10 seconds")
        time.sleep(5)
        Thread(target=show, args=()).start()






import typing

from fishy import web
from fishy.gui.notification import _give_notification_link

from tkinter import *
from tkinter.ttk import *

from fishy.helper.popup import PopUp

if typing.TYPE_CHECKING:
    from fishy.gui import GUI


def start_fullfisher_config(gui: 'GUI'):
    def save():
        gui._config.set("tesseract_dir", tesseract_entry.get(), False)
        gui._config.save_config()

    top = PopUp(save, gui._root, background=gui._root["background"])
    controls_frame = Frame(top)
    top.title("Config")

    Label(controls_frame, text="Tesseract Directory:").grid(row=0, column=0)
    tesseract_entry = Entry(controls_frame, justify=CENTER)
    tesseract_entry.insert(0, gui._config.get("tesseract_dir", ""))
    tesseract_entry.grid(row=0, column=1)

    controls_frame.pack(padx=(5, 5), pady=(5, 5))
    top.start()


def start_semifisher_config(gui: 'GUI'):
    def save():
        gui._config.set("action_key", action_key_entry.get(), False)
        gui._config.set("borderless", borderless.instate(['selected']), False)
        gui._config.save_config()

    top = PopUp(save, gui._root, background=gui._root["background"])
    controls_frame = Frame(top)
    top.title("Config")

    Label(controls_frame, text="Notification:").grid(row=0, column=0)

    gui._notify = IntVar(0)
    gui._notify_check = Checkbutton(controls_frame, command=lambda: _give_notification_link(gui),
                                    variable=gui._notify)
    gui._notify_check.grid(row=0, column=1)
    gui._notify_check['state'] = DISABLED
    is_subbed = web.is_subbed(gui._config.get('uid'))
    if is_subbed[1]:
        gui._notify_check['state'] = NORMAL
        gui._notify.set(is_subbed[0])

    Label(controls_frame, text="Fullscreen: ").grid(row=1, column=0, pady=(5, 5))
    borderless = Checkbutton(controls_frame, var=BooleanVar(value=gui._config.get("borderless")))
    borderless.grid(row=1, column=1)

    Label(controls_frame, text="Action Key:").grid(row=2, column=0)
    action_key_entry = Entry(controls_frame, justify=CENTER)
    action_key_entry.grid(row=2, column=1)
    action_key_entry.insert(0, gui._config.get("action_key", "e"))

    controls_frame.pack(padx=(5, 5), pady=(5, 5))
    top.start()



from tkinter import messagebox

import typing
if typing.TYPE_CHECKING:
    from fishy.gui import GUI


# noinspection PyProtectedMember
class GUIFuncs:
    def __init__(self, gui: 'GUI'):
        self.gui = gui

    def show_error(self, error):
        self.gui.call_in_thread(lambda: messagebox.showerror("ERROR", error))

    def bot_started(self, started):
        def func():
            self.gui._bot_running = started
            self.gui._start_button["text"] = self.gui._get_start_stop_text()

        self.gui.call_in_thread(func)

    def quit(self):
        def func():
            self.gui._root.destroy()

        self.gui.call_in_thread(func)



import logging
from fishy.engine import SemiFisherEngine
from fishy.engine.fullautofisher.engine import FullAuto


class EngineEventHandler:
    def __init__(self, config, gui_ref):
        self.event_handler_running = True
        self.event = []

        self.semi_fisher_engine = SemiFisherEngine(config, gui_ref)
        self.full_fisher_engine = FullAuto(config, gui_ref)

    def start_event_handler(self):
        while self.event_handler_running:
            while len(self.event) > 0:
                event = self.event.pop(0)
                event()

    def toggle_semifisher(self):
        self.event.append(self.semi_fisher_engine.toggle_start)

    def toggle_fullfisher(self):
        self.event.append(self.full_fisher_engine.toggle_start)

    def check_pixel_val(self):
        def func():
            if self.semi_fisher_engine.start:
                self.semi_fisher_engine.show_pixel_vals()
            else:
                logging.debug("Start the engine first before running this command")

        self.event.append(func)

    def quit(self):
        def func():
            self.semi_fisher_engine.start = False
            self.event_handler_running = False

        self.event.append(func)



from logging import StreamHandler

import typing

if typing.TYPE_CHECKING:
    from . import GUI


class GUIStreamHandler(StreamHandler):
    def __init__(self, gui):
        StreamHandler.__init__(self)
        self.gui = gui

    def emit(self, record):
        msg = self.format(record)
        self.gui.call_in_thread(lambda: _write_to_console(self.gui, msg))


def _write_to_console(root: 'GUI', msg):
    numlines = root._console.index('end - 1 line').split('.')[0]
    root._console['state'] = 'normal'
    if int(numlines) >= 50:  # delete old lines
        root._console.delete(1.0, 2.0)
    if root._console.index('end-1c') != '1.0':  # new line for each log
        root._console.insert('end', '\n')
    root._console.insert('end', msg)
    root._console.see("end")  # scroll to bottom
    root._console['state'] = 'disabled'



import logging
import os
import sys
import threading
import traceback
import webbrowser
from threading import Thread
from zipfile import ZipFile

from uuid import uuid1
from hashlib import md5

from win32com.client import Dispatch

import fishy
import winshell

from fishy import web
from . import Config


def not_implemented():
    logging.error("Not Implemented")


def open_web(website):
    """
    Opens a website on browser,
    uses multi-threading so that current thread doesnt get blocked
    :param website: url
    """
    logging.debug("opening web, please wait...")
    Thread(target=lambda: webbrowser.open(website, new=2)).start()


def initialize_uid(config: Config):
    if config.get("uid") is not None:
        return

    new_uid = _create_new_uid()
    if web.register_user(new_uid):
        config.set("uid", new_uid)
    else:
        logging.error("Couldn't register uid, some features might not work")


def _create_new_uid():
    """
    Creates a unique id for user
    """
    return md5(str(uuid1()).encode()).hexdigest()


def install_thread_excepthook():
    """
    Workaround for sys.excepthook thread bug
    https://bugs.python.org/issue1230540
    (https://sourceforge.net/tracker/?func=detail&atid=105470&aid=1230540&group_id=5470).
    Call once from __main__ before creating any threads.
    If using psyco, call psycho.cannotcompile(threading.Thread.run)
    since this replaces a new-style class method.
    """
    import sys
    run_old = threading.Thread.run

    # noinspection PyBroadException
    def run(*args, **kwargs):
        try:
            run_old(*args, **kwargs)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            sys.excepthook(*sys.exc_info())

    threading.Thread.run = run


def unhandled_exception_logging(*exc_info):
    text = "".join(traceback.format_exception(*exc_info))
    logging.error("Unhandled exception: %s", text)


def manifest_file(rel_path):
    """
    returns a file from the manifest files,
    used to get the files which are installed along with the scripts
    :param rel_path: relative path from `__init__.py`
    :return: abs path of the file
    """
    return os.path.join(os.path.dirname(fishy.__file__), rel_path)


def create_shortcut_first(c):
    if not c.get("shortcut_created", False):
        create_shortcut()
        c.set("shortcut_created", True)


# noinspection PyBroadException
def create_shortcut():
    """
    creates a new shortcut on desktop
    """
    try:
        desktop = winshell.desktop()
        path = os.path.join(desktop, "Fishybot ESO.lnk")

        shell = Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(path)
        shortcut.TargetPath = os.path.join(os.path.dirname(sys.executable), "python.exe")
        shortcut.Arguments = "-m fishy"
        shortcut.IconLocation = manifest_file("icon.ico")
        shortcut.save()

        logging.info("Shortcut created")
    except Exception:
        logging.error("Couldn't create shortcut")


# noinspection PyBroadException
def check_addon(name):
    """
    Extracts the addon from zip and installs it into the AddOn folder of eso
    """
    try:
        user = os.path.expanduser("~")
        addon_dir = os.path.join(user, "Documents", "Elder Scrolls Online", "live", "Addons")
        if not os.path.exists(os.path.join(addon_dir, name)):
            logging.info(f"{name} Addon not found, installing it...")
            with ZipFile(manifest_file(f"{name}.zip"), 'r') as z:
                z.extractall(path=addon_dir)
            logging.info("Please make sure you enable \"Allow outdated addons\" in-game")
    except Exception:
        logging.error("couldn't install addon, try doing it manually")


def restart():
    os.execl(sys.executable, *([sys.executable] + sys.argv))



import logging
import traceback
from functools import wraps


def uses_session(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        from .web import get_session
        if get_session(args[0]) is None:
            logging.error("Couldn't create a session")
            return None
        else:
            return f(*args, **kwargs)

    return wrapper


def fallback(default):
    def inner(f):
        # noinspection PyBroadException
        @wraps(f)
        def wrapper(*args, **kwargs):
            try:
                return f(*args, **kwargs)
            except Exception:
                traceback.print_exc()
                return default
        return wrapper

    return inner



import time
from tkinter import Toplevel


def center(win):
    win.update_idletasks()
    win.master.update_idletasks()
    width = win.winfo_width()
    height = win.winfo_height()

    offset_x = win.master.winfo_x() + win.master.winfo_width() // 2 - (width // 2)
    offset_y = win.master.winfo_y() + win.master.winfo_height() // 2 - (height // 2)

    win.geometry('{}x{}+{}+{}'.format(width, height, offset_x, offset_y))


class PopUp(Toplevel):
    def __init__(self, quit_callback, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.running = True
        self.quit_callback = quit_callback

    def quit_top(self):
        self.quit_callback()
        self.destroy()
        self.running = False

    def start(self):
        self.protocol("WM_DELETE_WINDOW", self.quit_top)
        self.grab_set()
        center(self)
        while self.running:
            self.update()
            time.sleep(0.01)
        self.grab_release()



import logging
import time
from tkinter import *
from tkinter.ttk import *

from pynput import keyboard
from pynput.keyboard import Key
from ttkthemes import ThemedTk

from fishy import helper
from fishy.gui import config_top

import typing

from fishy.helper import not_implemented

if typing.TYPE_CHECKING:
    from . import GUI


def _apply_theme(gui: 'GUI'):
    dark = gui._config.get("dark_mode", True)
    gui._root["theme"] = "equilux" if dark else "breeze"
    gui._console["background"] = "#707070" if dark else "#ffffff"
    gui._console["fg"] = "#ffffff" if dark else "#000000"


def _create(gui: 'GUI'):
    engines = {
        "Semi Fisher": [lambda: config_top.start_semifisher_config(gui), gui.engine.toggle_semifisher],
    }

    if gui._config.get('debug', False):
        engines["Full-Auto Fisher"] = [lambda: config_top.start_fullfisher_config(gui), gui.engine.toggle_fullfisher]

    def start_engine(label):
        gui._config.set("last_started", label)
        engines[label][1]()

    gui._root = ThemedTk(theme="equilux", background=True)
    gui._root.title("Fishybot for Elder Scrolls Online")

    gui._root.iconbitmap(helper.manifest_file('icon.ico'))

    # region menu
    menubar = Menu(gui._root)

    filemenu = Menu(menubar, tearoff=0)
    filemenu.add_command(label="Create Shortcut", command=helper.create_shortcut)

    def _toggle_mode():
        gui._config.set("dark_mode", not gui._config.get("dark_mode", True))
        gui._start_restart = True

    dark_mode_var = IntVar()
    dark_mode_var.set(int(gui._config.get('dark_mode', True)))
    filemenu.add_checkbutton(label="Dark Mode", command=_toggle_mode,
                             variable=dark_mode_var)

    menubar.add_cascade(label="File", menu=filemenu)

    debug_menu = Menu(menubar, tearoff=0)
    debug_menu.add_command(label="Check PixelVal",
                           command=lambda: gui.engine.check_pixel_val())

    debug_var = IntVar()
    debug_var.set(int(gui._config.get('debug', False)))

    def keep_console():
        gui._config.set("debug", bool(debug_var.get()))
        logging.debug("Restart to update the changes")

    debug_menu.add_checkbutton(label="Keep Console", command=keep_console, variable=debug_var)
    debug_menu.add_command(label="Restart", command=helper.restart)
    menubar.add_cascade(label="Debug", menu=debug_menu)

    help_menu = Menu(menubar, tearoff=0)
    help_menu.add_command(label="Troubleshoot Guide", command=not_implemented)
    help_menu.add_command(label="Need Help?", command=lambda: helper.open_web("http://discord.definex.in"))
    help_menu.add_command(label="Donate", command=lambda: helper.open_web("https://paypal.me/AdamSaudagar"))
    menubar.add_cascade(label="Help", menu=help_menu)

    gui._root.config(menu=menubar)
    # endregion

    # region console
    gui._console = Text(gui._root, state='disabled', wrap='none', background="#707070", fg="#ffffff")
    gui._console.pack(fill=BOTH, expand=True, pady=(15, 15), padx=(10, 10))
    gui._console.mark_set("sentinel", INSERT)
    gui._console.config(state=DISABLED)
    # endregion

    # region controls
    start_frame = Frame(gui._root)

    engine_var = StringVar(start_frame)
    labels = list(engines.keys())
    last_started = gui._config.get("last_started", labels[0])
    engine_select = OptionMenu(start_frame, engine_var, last_started, *labels)
    engine_select.pack(side=LEFT)

    config_button = Button(start_frame, text="⚙", width=0, command=lambda: engines[engine_var.get()][0]())
    config_button.pack(side=RIGHT)

    gui._start_button = Button(start_frame, text=gui._get_start_stop_text(), width=25,
                               command=lambda: start_engine(engine_var.get()))
    gui._start_button.pack(side=RIGHT)

    start_frame.pack(padx=(10, 10), pady=(5, 15), fill=X)
    # endregion

    _apply_theme(gui)
    gui._root.update()
    gui._root.minsize(gui._root.winfo_width() + 10, gui._root.winfo_height() + 10)

    def keyboard_listener(key):
        if key == Key.f9:
            gui.call_in_thread(lambda: start_engine(engine_var.get()))

    kb_listener = keyboard.Listener(
        on_press=keyboard_listener,
    )
    kb_listener.start()

    def set_destroy():
        gui._destroyed = True

    gui._root.protocol("WM_DELETE_WINDOW", set_destroy)
    gui._destroyed = False

    while True:
        gui._root.update()
        gui._clear_function_queue()
        if gui._start_restart:
            gui._root.destroy()
            gui._root.quit()
            gui._start_restart = False
            gui.create()
        if gui._destroyed:
            gui.engine.quit()
            break
        time.sleep(0.01)

    kb_listener.stop()



import ctypes
import logging
import os
import sys

import win32con
import win32gui

import fishy
from fishy import web, helper, gui
from fishy.engine.event_handler import EngineEventHandler
from fishy.gui import GUI
from fishy.helper import Config


# noinspection PyBroadException
def initialize(c: Config, window_to_hide):
    helper.create_shortcut_first(c)
    helper.initialize_uid(c)

    new_session = web.get_session(c)
    if new_session is None:
        logging.error("Couldn't create a session, some features might not work")
    print(f"created session {new_session}")

    try:
        is_admin = os.getuid() == 0
    except AttributeError:
        is_admin = ctypes.windll.shell32.IsUserAnAdmin() != 0

    if is_admin:
        logging.info("Running with admin privileges")

    try:
        helper.auto_upgrade()
    except Exception:
        pass

    if not c.get("debug", False):
        win32gui.ShowWindow(window_to_hide, win32con.SW_HIDE)
        helper.install_thread_excepthook()
        sys.excepthook = helper.unhandled_exception_logging

    helper.check_addon("ProvisionsChalutier")

    if c.get("debug", False):
        helper.check_addon("FooAddon")


def main():
    print("launching please wait...")

    window_to_hide = win32gui.GetForegroundWindow()
    c = Config()

    if not gui.check_eula(c):
        return

    bot = EngineEventHandler(c, lambda: gui_window)
    gui_window = GUI(c, lambda: bot)

    gui_window.start()

    logging.info(f"Fishybot v{fishy.__version__}")
    initialize(c, window_to_hide)

    bot.start_event_handler()


if __name__ == "__main__":
    main()



import webbrowser
from tkinter import *
from tkinter.ttk import *

from PIL import Image, ImageTk

from fishy import helper, web
from fishy.helper import Config

hyperlinkPattern = re.compile(r'\[(?P<title>.*?)\]\((?P<address>.*?)\)')


def check_eula(config):
    if not config.get("eula", False):
        _run_terms_window(config)
        return config.get("eula", False)

    return config.get("eula", False)


def _run_terms_window(config: Config):
    def accept():
        config.set("eula", True)
        root.destroy()

    def disable_enable_button():
        accept_button.config(state=NORMAL if check_value.get() else DISABLED)

    root = Tk()
    message = f'I agree to the [Terms of Service and Privacy Policy]({web.get_terms_page()})'
    root.title("EULA")
    root.resizable(False, False)
    root.iconbitmap(helper.manifest_file('icon.ico'))

    f = Frame(root)
    canvas = Canvas(f, width=300, height=200)
    canvas.pack()
    root.image = Image.open(helper.manifest_file('fishybot_logo.png')).resize((300, 200))
    root.image = ImageTk.PhotoImage(root.image)
    canvas.create_image(0, 0, anchor=NW, image=root.image)

    check_value = IntVar(0)

    g1 = Frame(f)
    Checkbutton(g1, command=disable_enable_button, variable=check_value).pack(side=LEFT)
    text = Text(g1, width=len(hyperlinkPattern.sub('\g<title>', message)),
                height=1, borderwidth=0, highlightthickness=0)
    text["background"] = root["background"]

    _format_hyper_link(text, message)
    text.config(state=DISABLED)
    text.pack(side=LEFT)
    g1.pack()

    f.pack(padx=(10, 10), pady=(20, 20))

    g2 = Frame(f)
    accept_button = Button(g2, text="Accept",
                           command=accept)
    accept_button.grid(row=0, column=0)
    Button(g2, text="Deny",
           command=lambda: root.destroy()).grid(row=0, column=1)
    g2.pack(pady=(5, 0))
    disable_enable_button()

    root.mainloop()


def _format_hyper_link(text, message):
    start = 0
    for index, match in enumerate(hyperlinkPattern.finditer(message)):
        groups = match.groupdict()
        text.insert("end", message[start: match.start()])
        # insert hyperlink tag here
        text.insert("end", groups['title'])
        text.tag_add(str(index),
                     "end-%dc" % (len(groups['title']) + 1),
                     "end", )
        text.tag_config(str(index),
                        foreground="blue",
                        underline=1)
        text.tag_bind(str(index),
                      "<Enter>",
                      lambda *a, **k: text.config(cursor="hand2"))
        text.tag_bind(str(index),
                      "<Leave>",
                      lambda *a, **k: text.config(cursor="arrow"))
        text.tag_bind(str(index),
                      "<Button-1>",
                      lambda x: webbrowser.open(groups['address']))
        start = match.end()
    else:
        text.insert("end", message[start:])



import cv2


def get_keypoint_from_image(img):
    """
    convert image int hsv
    creates a mask for brown color
    uses blob detection to find a blob in the mask
    filter the blobs to find the correct one

    :param img: rgb image
    :return: location of the pixel which is used to detect different fishing states
    """

    # Setup SimpleBlobDetector parameters.
    hsv_img = cv2.cvtColor(img, cv2.COLOR_RGB2HSV)
    lower = (99, 254, 100)
    upper = (100, 255, 101)
    mask = cv2.inRange(hsv_img, lower, upper)

    # Setup SimpleBlobDetector parameters.
    params = cv2.SimpleBlobDetector_Params()

    # Change thresholds
    params.minThreshold = 10
    params.maxThreshold = 255

    params.filterByColor = True
    params.blobColor = 255

    params.filterByCircularity = False
    params.filterByConvexity = False
    params.filterByInertia = False

    params.filterByArea = True
    params.minArea = 10.0

    detector = cv2.SimpleBlobDetector_create(params)

    # Detect blobs.
    key_points = detector.detect(mask)

    if len(key_points) <= 0:
        return None

    return int(key_points[0].pt[0]), int(key_points[0].pt[1])


class PixelLoc:
    """
    finds the pixel loc and store it
    """

    val = None

    @staticmethod
    def config():
        """
        Uses the game window to get an image of the game screen
        then uses `GetKeypointFromImage()` to find the ProvisionsChalutier pixel location
        :return: false if pixel loc not found
        """

        PixelLoc.val = (0, 0, 1, 1)

        return True



"""
fishing_event.py
Defines different fishing modes (states) which acts as state for state machine
also implements callbacks which is called when states are changed
"""
import logging
import time
from abc import abstractmethod, ABC

import pyautogui

from fishy import web

_fishCaught = 0
_totalFishCaught = 0
_stickInitTime = 0
_fish_times = []
_hole_start_time = 0
_FishingStarted = False


class FishEvent(ABC):
    @abstractmethod
    def on_enter_callback(self, previous_mode):
        pass

    @abstractmethod
    def on_exit_callback(self, current_mode):
        pass


class HookEvent(FishEvent):
    def __init__(self, action_key: str, collect_r: bool):
        self.action_key = action_key
        self.collect_r = collect_r

    def on_enter_callback(self, previous_mode):
        """
        called when the fish hook is detected
        increases the `fishCaught`  and `totalFishCaught`, calculates the time it took to catch
        presses e to catch the fish

        :param previous_mode: previous mode in the state machine
        """
        global _fishCaught, _totalFishCaught

        _fishCaught += 1
        _totalFishCaught += 1
        time_to_hook = time.time() - _stickInitTime
        _fish_times.append(time_to_hook)
        logging.info("HOOOOOOOOOOOOOOOOOOOOOOOK....... " + str(_fishCaught) + " caught " + "in " + str(
            round(time_to_hook, 2)) + " secs.  " + "Total: " + str(_totalFishCaught))
        pyautogui.press(self.action_key)

        if self.collect_r:
            time.sleep(0.1)
            pyautogui.press('r')
            time.sleep(0.1)

    def on_exit_callback(self, current_mode):
        pass


class LookEvent(FishEvent):
    """
    state when looking on a fishing hole
    """

    def __init__(self, action_key: str):
        self.action_key = action_key

    def on_enter_callback(self, previous_mode):
        """
        presses e to throw the fishing rod
        :param previous_mode: previous mode in the state machine
        """
        pyautogui.press(self.action_key)

    def on_exit_callback(self, current_mode):
        pass


class IdleEvent(FishEvent):
    """
    State when the fishing hole is depleted or the bot is doing nothing
    """

    def __init__(self, uid):
        """
        sets the flag to send notification on phone
        """
        self.uid = uid

    def on_enter_callback(self, previous_mode):
        """
        Resets the fishCaught counter and logs a message depending on the previous state
        :param previous_mode: previous mode in the state machine
        """
        global _fishCaught

        if _fishCaught > 0:
            web.send_hole_deplete(self.uid, _fishCaught, time.time() - _hole_start_time, _fish_times)
            _fishCaught = 0

        if previous_mode.name == "hook":
            logging.info("HOLE DEPLETED")
        else:
            logging.info("FISHING INTERRUPTED")

    def on_exit_callback(self, current_mode):
        pass


class StickEvent(FishEvent):
    """
    State when fishing is going on
    """

    def on_enter_callback(self, previous_mode):
        """
        resets the fishing timer
        :param previous_mode: previous mode in the state machine
        """
        global _stickInitTime, _hole_start_time, _fish_times, _FishingStarted

        _stickInitTime = time.time()
        _FishingStarted = True

        if _fishCaught == 0:
            _hole_start_time = time.time()
            _fish_times = []

    def on_exit_callback(self, current_mode):
        pass



import logging
from typing import List, Callable
import threading

from ttkthemes import ThemedTk

from fishy.engine.event_handler import EngineEventHandler
from fishy.gui.funcs import GUIFuncs
from . import main_gui
from .log_config import GUIStreamHandler
from fishy.helper import Config


class GUI:
    def __init__(self, config: Config, get_engine: Callable[[], EngineEventHandler]):
        """
        :param config: used to get and set configuration settings
        """
        self.funcs = GUIFuncs(self)
        self.get_engine = get_engine

        self._config = config
        self._start_restart = False
        self._destroyed = True
        self._log_strings = []
        self._function_queue: List[Callable] = []
        self._bot_running = False

        # UI items
        self._root: ThemedTk = None
        self._console = None
        self._start_button = None
        self._notify = None
        self._notify_check = None

        self._thread = threading.Thread(target=self.create, args=())

        root_logger = logging.getLogger('')
        root_logger.setLevel(logging.DEBUG)
        logging.getLogger('urllib3').setLevel(logging.WARNING)
        new_console = GUIStreamHandler(self)
        root_logger.addHandler(new_console)

    @property
    def engine(self):
        return self.get_engine()

    def create(self):
        main_gui._create(self)

    def start(self):
        self._thread.start()

    def _clear_function_queue(self):
        while len(self._function_queue) > 0:
            func = self._function_queue.pop(0)
            func()

    def call_in_thread(self, func: Callable):
        self._function_queue.append(func)

    def _get_start_stop_text(self):
        return "STOP (F9)" if self._bot_running else "START (F9)"



import sys

if "--local-server" in sys.argv:
    domain = "http://127.0.0.1:5000"
elif "--test-server" in sys.argv:
    domain = "https://fishyeso-test.herokuapp.com"
else:
    domain = "https://fishyeso.herokuapp.com"

user = domain + "/api/user"
notify = domain + "/api/notify"
subscription = domain + "/api/subscription/"
hole_depleted = domain + "/api/hole_depleted"
session = domain + "/api/session"
terms = domain + "/terms.html"


def get_notification_page(uid):
    return domain + f"?uid={uid}"


def get_terms_page():
    return terms
